load("@ytt:assert", "assert")
load("@ytt:data", "data")

SERVICE_TYPE_NODEPORT = "NodePort"
SERVICE_TYPE_LOADBALANCER = "LoadBalancer"

def validate_infrastructure_provider():
  data.values.infrastructure_provider in ("aws", "vsphere", "azure") or assert.fail("infrastructure provider should be either aws or vsphere or azure")
end

def validate_pinniped():
  data.values.tkg_cluster_role in ("management", "workload") or assert.fail("tkg_cluster_role must be provided to be either 'management' or 'workload'")
  data.values.infrastructure_provider in ("vsphere", "azure", "aws") or assert.fail("infrastructure_provider must be provided to be either 'vsphere', 'azure' or 'aws'")
  if data.values.identity_management_type:
    if is_mgmt_cluster():
      data.values.identity_management_type in ("oidc", "ldap") or assert.fail("identity_management_type for management clusters must be either 'oidc' or 'ldap'")
      validate_infrastructure_provider()
      validate_ldap_config()
    end
    if is_workload_cluster():
      data.values.identity_management_type in ("none", "oidc", "ldap") or assert.fail("identity_management_type for workload clusters be either 'none', 'oidc' or 'ldap'")
    end
  end
  if render_on_workload_cluster():
    data.values.pinniped.supervisor_svc_endpoint or assert.fail("the pinniped.supervisor_svc_endpoint must be provided")
    data.values.pinniped.supervisor_ca_bundle_data or assert.fail("the pinniped.supervisor_ca_bundle_data must be provided")
    (hasattr(data.values.pinniped, "concierge") and hasattr(data.values.pinniped.concierge, "audience") and data.values.pinniped.concierge.audience and data.values.pinniped.concierge.audience != "") or assert.fail("the pinniped.concierge.audience must be provided for workload clusters when identity_management_type is either 'oidc' or 'ldap'")
  end
end

def validate_ldap_config():
  if data.values.identity_management_type != "ldap":
    return
  end
  if data.values.pinniped.upstream_ldap.host:
    data.values.pinniped.upstream_ldap.bindDN or assert.fail("bindDN is required when using ldap identity management type")
    data.values.pinniped.upstream_ldap.bindPW or assert.fail("bindPW is required when using ldap identity management type")
    data.values.pinniped.upstream_ldap.userSearch.base or assert.fail("ldap userSearch base is required when using ldap identity management type")
    data.values.pinniped.upstream_ldap.userSearch.usernameAttr or assert.fail("ldap userSearch usernameAttr is required when using ldap identity management type")
    not data.values.pinniped.upstream_ldap.groupSearch.skipGroupRefresh or data.values.pinniped.upstream_ldap.groupSearch.skipGroupRefresh in (True, False) or assert.fail("skipGroupRefresh must be a boolean when specified")
  else:
    #! When not using the pinniped.ldap config, must be using the legacy dex config in an upgrade case. Do the old validations.
    data.values.dex.config.ldap.host or assert.fail("LDAP host is required when using ldap identity management type legacy settings")
    if data.values.dex.config.ldap.userSearch:
      data.values.dex.config.ldap.userSearch.baseDN or assert.fail("ldap userSearch enabled. baseDN is required when using ldap identity management type legacy settings")
    end
    if data.values.dex.config.ldap.groupSearch:
      data.values.dex.config.ldap.groupSearch.baseDN or assert.fail("ldap groupSearch enabled. baseDN is required when using ldap identity management type legacy settings")
    end
    #! These were optional for Dex but are now required for Pinniped, so validate them even in an upgrade case.
    data.values.dex.config.ldap.bindDN or assert.fail("bindDN is required when using ldap identity management type legacy settings")
    data.values.dex.config.ldap.bindPW or data.values.dex.config.ldap.BIND_PW_ENV_VAR or assert.fail("BIND_PW_ENV_VAR is required when using ldap identity management type legacy settings")
    (data.values.dex.config.ldap.groupSearch.userMatchers and len(data.values.dex.config.ldap.groupSearch.userMatchers) == 1) or assert.fail("must have exactly one group search user matchers when using ldap identity management type legacy settings")
    (data.values.dex.config.ldap.groupSearch.userMatchers[0].groupAttr and data.values.dex.config.ldap.groupSearch.userMatchers[0].groupAttr != "") or assert.fail("must have nonempty group search user matcher groupAttr when using ldap identity management type legacy settings")
    data.values.dex.config.ldap.groupSearch.userMatchers[0].groupAttr.lower() == "dn" and assert.fail("cannot use DN as the group search user matcher groupAttr when using ldap identity management type legacy settings")
  end
end

# vsphere, aws, azure currently supported.
def get_default_service_type():
  if data.values.infrastructure_provider == "vsphere":
    return SERVICE_TYPE_NODEPORT
  else:
    return SERVICE_TYPE_LOADBALANCER
  end
end

def get_pinniped_supervisor_service_type():
  if hasattr(data.values.pinniped, "supervisor") and hasattr(data.values.pinniped.supervisor, "service") and hasattr(data.values.pinniped.supervisor.service, "type") and data.values.pinniped.supervisor.service.type != None:
    return data.values.pinniped.supervisor.service.type
  else:
    return get_default_service_type()
  end
end

def is_pinniped_supervisor_service_type_NodePort():
  return get_pinniped_supervisor_service_type() == SERVICE_TYPE_NODEPORT
end

def is_mgmt_cluster():
  return data.values.tkg_cluster_role == "management"
end

def is_workload_cluster():
  return data.values.tkg_cluster_role == "workload"
end

# at present we ignore identity_management_type as a render gate for mgmt clusters
def render_on_mgmt_cluster():
  return is_mgmt_cluster()
end

def render_on_workload_cluster():
  return is_workload_cluster() and data.values.identity_management_type != "none"
end

# validate pinniped
validate_pinniped()
